/*  $Id: Event.java 1335 2012-08-25 13:27:49Z sdevaux $
 *
 * Tabs = 4 chars
 * copyright (C) 2002-2012 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */
package net.eduvax.util;

import java.util.Hashtable;
import java.util.Vector;

/** 
 * Timed event.
 * The message template table associated to this event class must be registered
 * as an application singleton. This table is a Hashtable&lt;String,String&gt;.
 * The key use to identify the table singleton in App service is this class' 
 * Class instance :
 * <code>
 * // register message table
 * Hashtable&lt;String,String&gt; table= .... ;
 * App.get().setSingleton(Event.class,table);
 * </code>
 * Do the same for each Event class' subclasse.
 * @see App singleton management service.
 */ 
public class Event {
    /** event timestamp. Default is system date in milliseconds */
    private long _timestamp;
    /** 
     * Get event timestamp.
     * @return the timestamp.
     */ 
    public long getTimeStamp() {
        return _timestamp;
    }
    /**
     * Get text message for the event.
     * @return textual description of the event. In case of no template in
     * message table, default string is event id followed by event arguments
     * separated by tab char.
     */ 
    public String getMessage() {
        String fmtMessage=null;
        @SuppressWarnings("unchecked")
        Hashtable<String,String> msgTable=
            (Hashtable<String,String>)App.get().getSingleton(getClass());
        if (msgTable!=null) 
        {
            fmtMessage=msgTable.get(_id);
        }
        String message;
        if (fmtMessage!=null) {
            message=String.format(fmtMessage,_args.toArray());
        }
        else if (_args!=null) {
            StringBuilder buf=new StringBuilder();
            for(int i=0;i<_args.size();i++) {
                buf.append("\t"+_args.elementAt(i));
            }
            message=buf.toString();
        }
        else {
            message="--";
        }
        return message;
    }
    /** 
     * Event's arguments. 
     * @djin.relation aggregation,-,*,Object
     */ 
    private Vector<Object> _args=null;
    /** 
     * Add argument to this event.
     * @param arg argument to add.
     */ 
    public void addArg(Object arg) {
        if (_args==null)
        {
            _args=new Vector<Object>();
        }
        _args.add(arg);
    }
    /**  
     * Retrieve an argument.
     * <p>Arguments expeceted regarding the event ID (non limitative, default 
     * use in this package, each package should subclass with its own Event 
     * class to avoid any conflict in customization) :</p>
     * - I_CMDSTART, I_CMDDONE, I_CMDFAILED, I_CMDREFUSED : command
     *   identification
     * - I_CMDSTEPOK, I_CMDSTEPKO :
     *   - command identification.
     *   - progression indicator (double value from 0 to 1)
     * @param index argument index (first is 0).
     * @return the requested argument or null if index overflow.
     */ 
    public Object getArg(int index) {
        if (index<_args.size() && index>=0)
        {
            return _args.elementAt(index);
        }
        else
        {
            return null;
        }
    }
    /** @return arguments count. */
    public int getArgCount()
    {
        return _args.size();
    }

    /** 
     * Get event class identification.
     * Subclasses should overrid to return their own class identification. 
     * @return "GE." (e.g. generic event) 
     */ 
    protected String getIdPrefix()
    {
        return "GE.";
    }

    /** 
     * Create new event.
     * @param id event identification.
     * @param level event severity.
     */ 
    public Event(String id, ELevel level)
    {
        _timestamp=App.getTimestamp();
        _id=getIdPrefix()+id;
        _level=level;
    }
    /** 
     * Create anti-dated timestamp.
     * @param ts timestamp (must be consistant with auto-dated event that
     * use system date in milliseconds to allow correct sort if required).
     * @param id event type identification
     * @param level event severity.
     */ 
    public Event(long ts,String id, ELevel level)
    {
        _timestamp=ts;
        _id=id;
        _level=level;
    }

    /** 
     * Get message full text description.  
     * String containing fields separated by tab char. List fo fields :
     * - timestamp
     * - level/severity
     * - id
     * - message as returned by getMessage() (may include sub fields 
     * also separated by tab char).  
     * @return string representation of this event.
     */
     @Override public String toString()
    {
        return _timestamp+"\t"+_level+"\t"+_id+"\t"+getMessage();
    }
    /** level / severity */
    private ELevel _level;
    /** Event severity classification */
    public enum ELevel { DEBUG, INFO2, INFO1 , INFO, WARNING, ERROR };
    /** @return event level / severity */
    public ELevel getLevel()
    {
        return _level;
    }
    /** 
     * String to level Enum utility. 
     * - err, error : ELevel.ERROR
     * - warn, warning : ELevel.WARNING
     * - info : ELevel.INFO
     * - info+, info1, infodet : ELevel.INFO1
     * - info++, info2 : ELevel.INFO2
     * - any other : ELevel.DEBUG
     * @param l string to convert, not case sensitive. 
     * @return level Enum.
     */ 
    public static ELevel parseLevel(String l)
    {
        ELevel res=ELevel.DEBUG;
        String level=l.toLowerCase();
        if ("err".equals(level)||"error".equals(level)) {
            res=ELevel.ERROR;
        }
        else if ("warn".equals(level)||"warning".equals(level)) {
            res=ELevel.WARNING;
        }
        else if("info".equals(level)) {
            res=ELevel.INFO;
        }
        else if ("info+".equals(level)||"info1".equals(level)
                ||"infodet".equals(level)) {
            res=ELevel.INFO1;
        }    
        else if ("info++".equals(level)||"info2".equals(level)) {
            res=ELevel.INFO2;
        }
        return res;
    }

    /** Event type identification */
    private String _id;
    /** 
     * @return event type name.
     */
    public String getId()
    {
        return _id;
    }
    /** 
     * type event name comparison.  
     * @param id candidate ident.
     * @return 
     * - true : candidate ident is equals to this event's id (including prefix)
     * - otherwise false.
     */
    public boolean checkId(String id)
    {
        return _id.equals((getIdPrefix()+id));
    }
    /** command start ident suffix. */
    public final static String I_CMDSTART="CmdStart";
    /** command step OK ident suffix. */
    public final static String I_CMDSTEPOK="CmdStepOK";
    /** command step failed ident suffix. */
    public final static String I_CMDSTEPKO="CmdStepKO";
    /** command succeed ident suffix. */
    public final static String I_CMDDONE="CmdDone";
    /** command failed ident suffix. */
    public final static String I_CMDFAILED="CmdFailed";
    /** command rejected ident suffix. */
    public final static String I_CMDREFUSED="CmdRefused";
}
