/*  $Id: WeakHashtable.java 1035 2010-06-15 13:08:33Z sdevaux $
 *
 * Tabs = 4 chars
 * copyright (C) 2002-2012 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */
package net.eduvax.util;

/** 
 * Alternate Observable/Observer utility.
 * Improvements from standar Observer/Observable :
 * - no more need send anonymous objects to be casted before use.
 * - only implement interface, no more ineritance restictions.
* <code>// Register an oberver to an observable.
* Observable observable;
* Observer observer;
* // ...
* observable.getObserverManager().add(observer);</code>
* <b>warning:</b> ObserverManager is memory less. a newly registered observable
* will not recieve notification sent to manager before the registration. 
* Only notification comming after full registration will be recieved 
* (unless some side effect of multi-threading).
 * <b>warning:</b> Java does not allow multiple inheritance of the same generic interface
 * even using different type parameters (C# does). Since this interface only define one
 * methode, there's a simple workaround :
 * <pre>
 * // can't declare:
 * // public class BiObs implements Observe&lt;A&gt;, Observer&lt;B&gt;  
 * // so doing this way :
 * public class BiObs {
 *		private void aChanged(A a) {
 *			// get changes of A.
 *		}
 *		private void bChanged(B b) {
 *			// get changes of B.
 *		}
 *		private Observer&lt;A&gt; aObs;
 *		private Observer&lt;B&gt; bObs;
 *		public BiObs(A a,B b) {
 *			aObs=new Observer&lt;A&gt;() {
 *				public void observableChanged(A a) {
 *					aChanged(a)
 *				}
 *			}
 *			bObs=new Observer&lt;B&gt;() {
 *				public void observableChanged(B b) {
 *					bChanged(b)
 *				}
 *			}
 *			a.getObsManager().add(aObs);
 *			b.getObsManager().add(bObs);
 *		}
 *		public void dispose() {
 *			a.getObsManager().remove(aObs);
 *			b.getObsManager().remove(bObs);
 *		}
 * }</pre>
 */
public interface Observer<T> 
{
	/** 
	 * Signal change of Observable.
	 * @param sender notification source.
	 */
	void observableChanged(T sender);
}
